-- ==========================================================================================
-- Operator_Overloading_1.adb
--
-- Demonstrates examples in 4.2.1 of Ada Distilled.
--   
-- Because the = operator for a record does a straight binary comparison, it is often a good
-- idea to create you own implementation of the = operator.  Sometimes you might want to give
-- the function a name such as Is_Equal instead of overloading the equality operator.  
-- 
-- ==========================================================================================
--
procedure Operator_Overloading_1 is

   type Inventory is                                     -- "=" is predefined for a record, but it does
      record                                             -- a full binary compare;  this is not what you
         Id          : Positive;                         -- usually want so you can define your own version
         Description : String (1 .. 60);                 -- of equality, greater-than, less-than, etc.
      end record; 

   function "=" (L, R : Inventory) return Boolean;       --  Specify an equality operator; operator overloading
   function Is_Equal (L, R : Inventory) return Boolean;  --  Named equality;  More readable than "="
   function ">" (L, R : Inventory) return Boolean;       --  Specify an greater-than operator

   -- An Implementation of  "=" Might Look Like This

   function "=" (L, R : Inventory) return Boolean is     --  1 Redefines an equal operator
   begin	                                             --  2 The usual begin statement 
      return L.Id = R.Id;	                           --  3 Compare only the ID part.
   end "=";	                                             --  4 Required scope terminator

   -- An Implementation of  ">" Might Look Like This

   function ">" (L, R : Inventory) return Boolean is     --  1 Redefines ">" operator
   begin	                                             --  2 The usual begin statement 
      return L.Id > R.Id;	                           --  3 Compare only the ID part.
   end ">";	                                             --  4 Required scope terminator

   function Is_Equal (L, R : Inventory) return Boolean is   --  1 Redefines an equal operator with Is_Equal
   begin	                                                --  2 The usual begin statement 
      return L.Id = R.Id;	                              --  3 Compare only the ID part.
   end Is_Equal;	                                          --  4 Required scope terminator



begin
   null;                                                 -- Must have at least one statement between begin and end
end Operator_Overloading_1;